@@ -139,30 +139,30 @@ def cli_get_cache(user_function):
 
     def decorating_function(*args):
         cli = args[0]
+        nocache = cli.nocache
+        if nocache:
+            cli.log.debug("cache disabled.")
+            return log_exec_time(user_function, *args)
         url = cli.get_full_url(args[1])
         cli.log.debug("cache url : " + url)
         key = hashlib.sha256(url + "|" + cli.user).hexdigest()
         cli.log.debug("key: " + key)
         cachefile = cachedir + "/" + key
         cache_time = cli.cache_time
-        nocache = cli.nocache
         res = None
-        if nocache:
-            log.debug("cache disabled.")
-            return log_exec_time(user_function, *args)
         if os.path.isfile(cachefile):
             file_time = os.stat(cachefile).st_mtime
             form = "{da:%Y-%m-%d %H:%M:%S}"
-            log.debug("cached data : " + str(
+            cli.log.debug("cached data : " + str(
                 form.format(da=datetime.datetime.fromtimestamp(file_time))))
             if time.time() - cache_time > file_time:
-                log.debug("refreshing cached data.")
+                cli.log.debug("refreshing cached data.")
                 res = get_data(user_function, cachefile, *args)
             if not res:
                 try:
                     res = load_data(cachefile)
                 except ValueError as ex:
-                    log.debug("error : " + str(ex))
+                    cli.log.debug("error : " + str(ex))
         if not res:
             res = get_data(user_function, cachefile, *args)
         return res
@@ -418,7 +418,8 @@ class CoreCli(object):
         url = self.get_full_url(url)
         self.log.debug("create url : " + url)
         # Building request
-        post_data = json.dumps(data).encode("UTF-8")
+        post_data = json.dumps(data)
+        post_data = post_data.encode("UTF-8")
         request = urllib2.Request(url, post_data)
         request.add_header('Content-Type', 'application/json; charset=UTF-8')
         request.add_header('Accept', 'application/json')
@@ -446,24 +447,29 @@ class CoreCli(object):
                         "time": self.last_req_time})
         return ret
 
-    def upload(self, file_path, url, description=None):
+    def upload(self, file_path, url, description=None, file_name=None,
+               progress_bar=True):
         self.last_req_time = None
         url = self.get_full_url(url)
         self.log.debug("upload url : " + url)
         # Generating datas and headers
         file_size = os.path.getsize(file_path)
         self.log.debug("file_path is : " + file_path)
-        file_name = os.path.basename(file_path)
+        if not file_name:
+            file_name = os.path.basename(file_path)
         self.log.debug("file_name is : " + file_name)
         if file_size <= 0:
             msg = "The file '%(filename)s' can not be uploaded \
 because its size is less or equal to zero." % {"filename": str(file_name)}
             raise LinShareException("-1", msg)
-        widgets = [FileTransferSpeed(), ' <<<', Bar(), '>>> ',
-                   Percentage(), ' ', ETA()]
-        pbar = ProgressBar(widgets=widgets, maxval=file_size)
-        stream = FileWithCallback(file_path, 'rb', pbar.update,
-                                  file_size, file_path)
+        pbar = None
+        stream = file(file_path, 'rb')
+        if progress_bar:
+            widgets = [FileTransferSpeed(), ' <<<', Bar(), '>>> ',
+                       Percentage(), ' ', ETA()]
+            pbar = ProgressBar(widgets=widgets, maxval=file_size)
+            stream = FileWithCallback(file_path, 'rb', pbar.update,
+                                      file_size, file_path)
         post = poster.encode.MultipartParam("file", filename=file_name,
                                             fileobj=stream)
         params = [post,]
@@ -474,7 +480,8 @@ because its size is less or equal to zero." % {"filename": str(file_name)}
         request = urllib2.Request(url, datagen, headers)
         request.add_header('Accept', 'application/json')
         # request start
-        pbar.start()
+        if pbar:
+            pbar.start()
         starttime = datetime.datetime.now()
         resultq = None
         try:
@@ -499,7 +506,8 @@ because its size is less or equal to zero." % {"filename": str(file_name)}
             self.log.debug("Server error message : " + str(msg))
             # request end
             endtime = datetime.datetime.now()
-            pbar.finish()
+            if pbar:
+                pbar.finish()
             self.last_req_time = str(endtime - starttime)
             if ex.code == 502:
                 self.log.warn(
@@ -516,45 +524,47 @@ because its size is less or equal to zero." % {"filename": str(file_name)}
             raise LinShareException(code, msg)
         # request end
         endtime = datetime.datetime.now()
-        pbar.finish()
+        if pbar:
+            pbar.finish()
         self.last_req_time = str(endtime - starttime)
         self.log.debug("upload url : %(url)s : request time : %(time)s",
                        {"url": url,
                         "time": self.last_req_time})
         return json_obj
 
-    def download(self, uuid, url):
+    def download(self, uuid, url, forced_file_name=None,
+                 progress_bar=True, chunk_size=256):
         """ download a file from LinShare using its rest api.
 This method could throw exceptions like urllib2.HTTPError."""
         self.last_req_time = None
         url = self.get_full_url(url)
         self.log.debug("download url : " + url)
-
         # Building request
         request = urllib2.Request(url)
         #request.add_header('Content-Type', 'application/json; charset=UTF-8')
         request.add_header('Accept', 'application/json;charset=UTF-8')
-
         # request start
         starttime = datetime.datetime.now()
-
         # doRequest
         resultq = urllib2.urlopen(request)
-
         code = resultq.getcode()
         file_name = uuid
         self.log.debug("ret code : '" + str(code) + "'")
         if code == 200:
             content_lenth = resultq.info().getheader('Content-Length')
             if not content_lenth:
-                self.log.error("No content lengh header found !")
+                msg = "No content lengh header found !"
+                self.log.error(msg)
                 result = resultq.read()
-                print result
-                return
+                self.log.error(result)
+                raise LinShareException("-1", msg)
             file_size = int(resultq.info().getheader('Content-Length').strip())
-            content_dispo = resultq.info().getheader('Content-disposition')
-            content_dispo = content_dispo.strip()
-            file_name = extract_file_name(content_dispo)
+            if forced_file_name:
+                file_name = forced_file_name
+            else:
+                content_dispo = resultq.info().getheader('Content-disposition')
+                content_dispo = content_dispo.strip()
+                file_name = extract_file_name(content_dispo)
             if os.path.isfile(file_name):
                 cpt = 1
                 while 1:
@@ -562,19 +572,17 @@ This method could throw exceptions like urllib2.HTTPError."""
                         file_name += "." + str(cpt)
                         break
                     cpt += 1
-
-            widgets = [FileTransferSpeed(), ' <<<', Bar(), '>>> ',
-                       Percentage(), ' ', ETA()]
-            pbar = ProgressBar(widgets=widgets, maxval=file_size)
-            stream = FileWithCallback(file_name, 'w', pbar.update,
-                                      file_size, file_name)
-            pbar.start()
-
-            chunk_size = 8192
-            chunk_size = 4096
-            chunk_size = 2048
-            chunk_size = 1024
-            chunk_size = 256
+            stream = None
+            pbar = None
+            if progress_bar:
+                widgets = [FileTransferSpeed(), ' <<<', Bar(), '>>> ',
+                           Percentage(), ' ', ETA()]
+                pbar = ProgressBar(widgets=widgets, maxval=file_size)
+                stream = FileWithCallback(file_name, 'w', pbar.update,
+                                          file_size, file_name)
+                pbar.start()
+            else:
+                stream = file(file_name, 'w')
             while 1:
                 chunk = resultq.read(chunk_size)
                 if not chunk:
@@ -582,8 +590,8 @@ This method could throw exceptions like urllib2.HTTPError."""
                 stream.write(chunk)
             stream.flush()
             stream.close()
-            pbar.finish()
-
+            if pbar:
+                pbar.finish()
         # request end
         endtime = datetime.datetime.now()
         self.last_req_time = str(endtime - starttime)
